// macros.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Object* pushqExpander(Symbol* variable, Object* value) {
  // Push 'value' onto the cons list 'variable'.
  return (listO(3, SYM_MACROS_STELLA_SETQ, variable, cons(listO(3, SYM_MACROS_STELLA_CONS, value, cons(variable, NIL)), NIL)));
}

Object* popqExpander(Symbol* variable) {
  // Pops a value from the cons list 'variable'.
  { Symbol* valuevar = localGensym("HEAD");

    return (listO(5, SYM_MACROS_STELLA_VRLET, cons(cons(valuevar, cons(listO(3, SYM_MACROS_STELLA_VALUE, variable, NIL), NIL)), NIL), listO(3, SYM_MACROS_STELLA_SETQ, variable, cons(listO(3, SYM_MACROS_STELLA_REST, variable, NIL), NIL)), valuevar, NIL));
  }
}

Object* pushfExpander(Cons* place, Object* value) {
  // Push 'value' onto the cons list 'place'.
  { Cons* placecopy = ((Cons*)(copyConsTree(place)));

    return (listO(3, SYM_MACROS_STELLA_SETF, place, cons(listO(3, SYM_MACROS_STELLA_CONS, value, cons(placecopy, NIL)), NIL)));
  }
}

Object* pushqNewExpander(Symbol* variable, Object* value) {
  // Push 'value' onto the cons list 'variable', if its
  // not there already.
  return (listO(4, SYM_MACROS_STELLA_WHEN, listO(3, SYM_MACROS_STELLA_NOT, listO(4, SYM_MACROS_STELLA_MEMBERp, SYM_MACROS_STELLA_VARIABLE, SYM_MACROS_STELLA_VALUE, NIL), NIL), listO(3, SYM_MACROS_STELLA_SETQ, variable, cons(listO(3, SYM_MACROS_STELLA_CONS, value, cons(variable, NIL)), NIL)), NIL));
}

Object* stringQuoteExpander(Object* tree) {
  // Return a parse tree that evaluates to 'tree' at
  // run-time.  'tree' is encoded as a string within the parse tree.
  return (listO(3, SYM_MACROS_STELLA_UNSTRINGIFY, wrapString(stringify(tree)), NIL));
}

Object* IIExpander(Object* place, Cons* increment) {
  // Increment the value of `place' and return the result.
  // `place' can be either a variable name or a slot reference.  Increment by
  // the optional `increment' (which can be a float) or 1 otherwise.
  { Object* incr = increment->value;

    if (!(symbolP(place) ||
        consP(place))) {
      if (!((boolean)(place))) {
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signalTranslationError();
          if (!(suppressWarningsP())) {
            printErrorContext(">> ERROR: ", STANDARD_ERROR);
            *(STANDARD_ERROR->nativeStream) << endl << " " << "Missing place in `++' expression" << "." << endl;
          }
        }
      }
      else {
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signalTranslationError();
          if (!(suppressWarningsP())) {
            printErrorContext(">> ERROR: ", STANDARD_ERROR);
            *(STANDARD_ERROR->nativeStream) << endl << " " << "Illegal place in `++' expression: " << "`" << deUglifyParseTree(place) << "'" << "." << endl;
          }
        }
      }
      return (NIL);
    }
    if (!((boolean)(incr))) {
      incr = ONE_WRAPPER;
    }
    return ((symbolP(place) ? listO(3, SYM_MACROS_STELLA_SETQ, place, cons(listO(3, SYM_MACROS_STELLA_i, place, cons(incr, NIL)), NIL)) : listO(3, SYM_MACROS_STELLA_SETF, place, cons(listO(3, SYM_MACROS_STELLA_i, copyConsTree(place), cons(incr, NIL)), NIL))));
  }
}

Object* Expander(Object* place, Cons* decrement) {
  // Decrement the value of `place' and return the result.
  // `place' can be either a variable name or a slot reference.  Decrement by
  // the optional `decrement' (which can be a float) or 1 otherwise.
  { Object* decr = decrement->value;

    if (!(symbolP(place) ||
        consP(place))) {
      if (!((boolean)(place))) {
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signalTranslationError();
          if (!(suppressWarningsP())) {
            printErrorContext(">> ERROR: ", STANDARD_ERROR);
            *(STANDARD_ERROR->nativeStream) << endl << " " << "Missing place in `--' expression" << "." << endl;
          }
        }
      }
      else {
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signalTranslationError();
          if (!(suppressWarningsP())) {
            printErrorContext(">> ERROR: ", STANDARD_ERROR);
            *(STANDARD_ERROR->nativeStream) << endl << " " << "Illegal place in `--' expression: " << "`" << deUglifyParseTree(place) << "'" << "." << endl;
          }
        }
      }
      return (NIL);
    }
    if (!((boolean)(decr))) {
      decr = ONE_WRAPPER;
    }
    return ((symbolP(place) ? listO(3, SYM_MACROS_STELLA_SETQ, place, cons(listO(3, SYM_MACROS_STELLA__, place, cons(decr, NIL)), NIL)) : listO(3, SYM_MACROS_STELLA_SETF, place, cons(listO(3, SYM_MACROS_STELLA__, copyConsTree(place), cons(decr, NIL)), NIL))));
  }
}

Object* oneIExpander(Object* expression) {
  // Add 1 to 'expression' and return the result.
  return (listO(3, SYM_MACROS_STELLA_i, expression, cons(wrapInteger(1), NIL)));
}

Object* oneExpander(Object* expression) {
  // Subtract 1 from 'expression' and return the result.
  return (listO(3, SYM_MACROS_STELLA__, expression, cons(wrapInteger(1), NIL)));
}

Object* setqPExpander(Symbol* variable, Cons* expression) {
  // Assign 'variable' the result of evaluating 'expression', 
  // and return TRUE if 'expression' is not NULL else return FALSE.
  return (listO(5, SYM_MACROS_STELLA_VRLET, NIL, listO(3, SYM_MACROS_STELLA_SETQ, variable, cons(expression, NIL)), listO(3, SYM_MACROS_STELLA_DEFINEDp, variable, NIL), NIL));
}

Object* firstDefinedExpander(Cons* forms) {
  // Return the result of the first form in `forms' whose value is defined
  // or NULL otherwise.
  switch (forms->length()) {
    case 0: 
      return (SYM_MACROS_STELLA_NULL);
    case 1: 
      return (forms->value);
    case 2: 
      if (symbolP(forms->value)) {
        return (listO(4, SYM_MACROS_STELLA_CHOOSE, listO(3, SYM_MACROS_STELLA_DEFINEDp, forms->value, NIL), forms->value, cons(forms->rest->value, NIL)));
      }
      else {
        { Symbol* letvariable = localGensym("TEMP");

          return (listO(4, SYM_MACROS_STELLA_VRLET, cons(cons(letvariable, cons(forms->value, NIL)), NIL), listO(4, SYM_MACROS_STELLA_CHOOSE, listO(3, SYM_MACROS_STELLA_DEFINEDp, letvariable, NIL), letvariable, cons(forms->rest->value, NIL)), NIL));
        }
      }
    break;
    default:
      return (listO(3, SYM_MACROS_STELLA_FIRST_DEFINED, forms->value, cons(cons(SYM_MACROS_STELLA_FIRST_DEFINED, forms->rest->concatenate(NIL, 0)), NIL)));
  }
}

Object* allDefinedPExpander(Cons* forms) {
  // Evaluate each of the forms in 'forms', and
  // return TRUE if none of them are NULL.
  { Cons* tests = NIL;

    { Object* f = NULL;
      Cons* iter000 = forms;
      Cons* collect000 = NULL;

      while (!(iter000 == NIL)) {
        f = iter000->value;
        iter000 = iter000->rest;
        if (!((boolean)(collect000))) {
          {
            collect000 = cons(listO(3, SYM_MACROS_STELLA_DEFINEDp, f, NIL), NIL);
            if (tests == NIL) {
              tests = collect000;
            }
            else {
              addConsToEndOfConsList(tests, collect000);
            }
          }
        }
        else {
          {
            collect000->rest = cons(listO(3, SYM_MACROS_STELLA_DEFINEDp, f, NIL), NIL);
            collect000 = collect000->rest;
          }
        }
      }
    }
    return (cons(SYM_MACROS_STELLA_AND, tests->concatenate(NIL, 0)));
  }
}

Object* eitherExpander(Object* value1, Object* value2) {
  // If 'value1' is defined, return that, else return 'value2'.
  return (listO(3, SYM_MACROS_STELLA_FIRST_DEFINED, value1, cons(value2, NIL)));
}

Object* collectExpander(Symbol* collectvariable, Cons* body) {
  // Use a VRLET to collect values.  Input has the form
  // '(collect <x> in <expression> where (<test> <x>))'.
  { Symbol* collection = gensym("V");

    return (listO(5, SYM_MACROS_STELLA_VRLET, cons(cons(collection, cons(SYM_MACROS_STELLA_NIL, NIL)), NIL), listO(3, SYM_MACROS_STELLA_FOREACH, collectvariable, body->concatenate(listO(3, SYM_MACROS_STELLA_COLLECT, collectvariable, listO(3, SYM_MACROS_STELLA_INTO, collection, NIL)), 0)), collection, NIL));
  }
}

Object* withPermanentObjectsExpander(Cons* body) {
  // Allocate 'permanent' (as opposed to 'transient')
  // objects within the scope of this declaration.
  return (listO(3, SYM_MACROS_STELLA_SPECIAL, cons(listO(3, SYM_MACROS_STELLA_oTRANSIENTOBJECTSpo, SYM_MACROS_STELLA_FALSE, NIL), NIL), body->concatenate(NIL, 0)));
}

Object* withTransientObjectsExpander(Cons* body) {
  // Allocate 'transient' (as opposed to 'permanent')
  // objects within the scope of this declaration.
  // CAUTION: The default assumption is the allocation of permanent objects.
  // The scope of `with-transient-objects' should be as small as possible, and
  // the user has to make sure that code that wasn't explicitly written to
  // account for transient objects will continue to work correctly.
  return (listO(3, SYM_MACROS_STELLA_SPECIAL, cons(listO(3, SYM_MACROS_STELLA_oTRANSIENTOBJECTSpo, SYM_MACROS_STELLA_TRUE, NIL), NIL), body->concatenate(NIL, 0)));
}

Object* ignoreExpander(Cons* variables) {
  // Ignore unused 'variables' with NoOp 'setq' statements.
  { Cons* ignoretrees = NIL;

    { Object* variable = NULL;
      Cons* iter000 = variables;
      Cons* collect000 = NULL;

      while (!(iter000 == NIL)) {
        variable = iter000->value;
        iter000 = iter000->rest;
        if (!((boolean)(collect000))) {
          {
            collect000 = cons(listO(3, SYM_MACROS_STELLA_SETQ, variable, cons(variable, NIL)), NIL);
            if (ignoretrees == NIL) {
              ignoretrees = collect000;
            }
            else {
              addConsToEndOfConsList(ignoretrees, collect000);
            }
          }
        }
        else {
          {
            collect000->rest = cons(listO(3, SYM_MACROS_STELLA_SETQ, variable, cons(variable, NIL)), NIL);
            collect000 = collect000->rest;
          }
        }
      }
    }
    return (prognify(ignoretrees));
  }
}

Object* onlyIfExpander(Object* test, Object* expression) {
  // If 'test' is TRUE, return the result of evaluating 
  // 'expression'.
  return (listO(3, SYM_MACROS_STELLA_CHOOSE, test, cons(expression, cons(SYM_MACROS_STELLA_NULL, NIL))));
}

Object* phaseToIntegerExpander(Keyword* startuptimephase) {
  // Expands into the integer representing 'startupTimePhase'.
  return (wrapInteger(encodeStartupTimePhase(startuptimephase)));
}

Object* ifOutputLanguageExpander(Keyword* language, Object* thenform, Object* elseform) {
  // Expand to 'thenForm' if the current translator output
  // language equals 'language'.  Otherwise, expand to 'elseForm'.  This can
  // be used to conditionally translate Stella code.
  if (language == oTRANSLATOROUTPUTLANGUAGEo.get()) {
    return (((thenform == SYM_MACROS_STELLA_NULL) ? NULL : thenform));
  }
  else {
    return (((elseform == SYM_MACROS_STELLA_NULL) ? NULL : elseform));
  }
}

Object* ifStellaFeatureExpander(Keyword* feature, Object* thenform, Object* elseform) {
  // Expand to 'thenForm' if 'feature' is a currently enabled
  // STELLA environment feature.  Otherwise, expand to 'elseForm'.  This can
  // be used to conditionally translate Stella code.
  if (oCURRENT_ENVIRONMENT_FEATURESo->memberP(feature)) {
    return (((thenform == SYM_MACROS_STELLA_NULL) ? NULL : thenform));
  }
  else {
    return (((elseform == SYM_MACROS_STELLA_NULL) ? NULL : elseform));
  }
}

Object* withinWorldExpander(Object* worldform, Cons* body) {
  // Execute `body' within the world resulting from `worldForm'.
  return (listO(4, SYM_MACROS_STELLA_SPECIAL, cons(listO(3, SYM_MACROS_STELLA_oCONTEXTo, worldform, NIL), NIL), listO(7, SYM_MACROS_STELLA_SAFETY, wrapInteger(3), listO(4, SYM_MACROS_STELLA_EQLp, listO(3, SYM_MACROS_STELLA_BASE_MODULE, listO(4, SYM_MACROS_STELLA_CAST, SYM_MACROS_STELLA_oCONTEXTo, SYM_MACROS_STELLA_WORLD, NIL), NIL), SYM_MACROS_STELLA_oMODULEo, NIL), wrapString("within-world: The base module of world "), SYM_MACROS_STELLA_oCONTEXTo, wrapString(" does not match the current module."), NIL), body->concatenate(NIL, 0)));
}

Object* withinContextExpander(Object* contextform, Cons* body) {
  // Execute `body' within the context resulting from `contextForm'.
  return (listO(3, SYM_MACROS_STELLA_SPECIAL, listO(3, listO(3, SYM_MACROS_STELLA_oCONTEXTo, contextform, NIL), listO(3, SYM_MACROS_STELLA_oMODULEo, listO(3, SYM_MACROS_STELLA_BASE_MODULE, SYM_MACROS_STELLA_oCONTEXTo, NIL), NIL), NIL), body->concatenate(NIL, 0)));
}

Object* withinModuleExpander(Object* moduleform, Cons* body) {
  // Execute `body' within the module resulting from `moduleForm'.
  // `*module*' is an acceptable `moduleForm'.  It will locally rebind 
  // `*module*' and `*context*' and shield the outer bindings from changes.
  return (listO(3, SYM_MACROS_STELLA_SPECIAL, listO(3, listO(3, SYM_MACROS_STELLA_oMODULEo, moduleform, NIL), listO(3, SYM_MACROS_STELLA_oCONTEXTo, SYM_MACROS_STELLA_oMODULEo, NIL), NIL), body->concatenate(NIL, 0)));
}

Object* COERCE_AREST_TO_CONS_EXPANDER(Symbol* restvariable) {
  // Coerce the argument list variable `restVariable' into a CONS
  // list containing all its elements (uses argument list iteration to do so).  If
  // `restVariable' already is a CONS due to argument listification, this is a no-op.
  { StandardObject* dummy1;

    if (passVariableArgumentsAsListP(oMETHODBEINGWALKEDo.get())) {
      return (sysTree(walkWithoutTypeTree(restvariable), yieldListifiedVariableArgumentsType(oMETHODBEINGWALKEDo.get()), dummy1));
    }
    else {
      { Symbol* argvar = localGensym("ARG");
        Symbol* listvar = localGensym("ARGLIST");

        return (listO(5, SYM_MACROS_STELLA_VRLET, cons(cons(listvar, cons(yieldListifiedVariableArgumentsType(oMETHODBEINGWALKEDo.get()), cons(SYM_MACROS_STELLA_NIL, NIL))), NIL), listO(3, SYM_MACROS_STELLA_FOREACH, argvar, listO(3, SYM_MACROS_STELLA_IN, restvariable, listO(3, SYM_MACROS_STELLA_COLLECT, argvar, listO(3, SYM_MACROS_STELLA_INTO, listvar, NIL)))), listvar, NIL));
      }
    }
  }
}

Object* withSystemDefinitionExpander(Object* systemnameexpression, Cons* body) {
  // Set *currentSystemDefinition* to the system definition named 'system'.
  // Set *currentSystemDefinitionSubdirectory* to match. Execute 'body' within
  // that scope.
  return (listO(4, SYM_MACROS_STELLA_SPECIAL, listO(3, listO(3, SYM_MACROS_STELLA_oCURRENTSYSTEMDEFINITIONo, listO(3, SYM_MACROS_STELLA_GET_SYSTEM_DEFINITION, systemnameexpression, NIL), NIL), listO(3, SYM_MACROS_STELLA_oCURRENTSYSTEMDEFINITIONSUBDIRECTORYo, listO(4, SYM_MACROS_STELLA_ONLY_IF, listO(3, SYM_MACROS_STELLA_DEFINEDp, SYM_MACROS_STELLA_oCURRENTSYSTEMDEFINITIONo, NIL), listO(3, SYM_MACROS_STELLA_DIRECTORY, SYM_MACROS_STELLA_oCURRENTSYSTEMDEFINITIONo, NIL), NIL), NIL), NIL), listO(5, SYM_MACROS_STELLA_IF, listO(3, SYM_MACROS_STELLA_DEFINEDp, SYM_MACROS_STELLA_oCURRENTSYSTEMDEFINITIONo, NIL), cons(SYM_MACROS_STELLA_PROGN, body->concatenate(NIL, 0)), listO(4, SYM_MACROS_STELLA_WARN, wrapString("Can't find a system named "), systemnameexpression, cons(SYM_MACROS_STELLA_EOL, NIL)), NIL), NIL));
}

Object* defmainExpander(Cons* varlist, Cons* body) {
  // Defines a function called MAIN which will have the appropriate
  // signature for the target translation language.  The signature will be:
  //   C++:   public static int main (int v1, char** v2) {<body>}
  // }}  Java:  public static void main (String [] v2) {<body>}
  //   Lisp:  (defun main (&rest args) <body>)
  // The argument `varList' must have two symbols, which will be the names for the
  // INTEGER argument count and an array of STRINGs with the argument values.  It
  // can also be empty to indicate that no command line arguments will be handled.
  // The startup function for the containing system will automatically be called
  // before `body' is executed unless the option :STARTUP-SYSTEM? was supplied as
  // FALSE.  There can only be one DEFMAIN per module.
  { boolean processcmdlineargsP = !(varlist == NIL);
    Symbol* v1Name = ((Symbol*)(varlist->value));
    Symbol* v2Name = ((Symbol*)(varlist->rest->value));
    Cons* bodywithheader = cons(NULL, body);

    { PropertyList* self000 = newPropertyList();

      self000->thePlist = extractOptions(bodywithheader, NULL);
      { PropertyList* options = self000;
        Cons* startupfunctioncall = ((((boolean)(oCURRENTSYSTEMDEFINITIONo.get())) &&
            (!(options->lookup(KWD_MACROS_STARTUP_SYSTEMp) == SYM_MACROS_STELLA_FALSE))) ? cons(cons(systemStartupFunctionSymbol(oCURRENTSYSTEMDEFINITIONo.get()), NIL), NIL) : NIL);

        options->removeAt(KWD_MACROS_STARTUP_SYSTEMp);
        options->removeAt(KWD_MACROS_PUBLICp);
        body = options->thePlist->concatenate(bodywithheader->rest, 0);
        if (!processcmdlineargsP) {
          if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_MACROS_COMMON_LISP) {
            return (listO(6, SYM_MACROS_STELLA_DEFUN, SYM_MACROS_STELLA_MAIN, NIL, KWD_MACROS_PUBLICp, SYM_MACROS_STELLA_TRUE, startupfunctioncall->concatenate(body->concatenate(NIL, 0), 0)));
          }
          else if ((oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_MACROS_CPP) ||
              ((oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_MACROS_CPP_STANDALONE) ||
               (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_MACROS_IDL))) {
            return (listO(6, SYM_MACROS_STELLA_DEFUN, listO(3, SYM_MACROS_STELLA_MAIN, SYM_MACROS_STELLA_INTEGER, NIL), NIL, KWD_MACROS_PUBLICp, SYM_MACROS_STELLA_TRUE, startupfunctioncall->concatenate(body->concatenate(cons(listO(3, SYM_MACROS_STELLA_RETURN, wrapInteger(1), NIL), NIL), 0), 0)));
          }
          else if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_MACROS_JAVA) {
            return (listO(6, SYM_MACROS_STELLA_DEFUN, SYM_MACROS_STELLA_MAIN, cons(cons(localGensym("ARGV"), cons(listO(5, SYM_MACROS_STELLA_ARRAY, NIL, SYM_MACROS_STELLA_OF, SYM_MACROS_STELLA_STRING, NIL), NIL)), NIL), KWD_MACROS_PUBLICp, SYM_MACROS_STELLA_TRUE, startupfunctioncall->concatenate(body->concatenate(NIL, 0), 0)));
          }
          else {
          }
        }
        else {
          if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_MACROS_COMMON_LISP) {
            return (listO(7, SYM_MACROS_STELLA_DEFUN, SYM_MACROS_STELLA_MAIN, listO(3, SYM_MACROS_STELLA_aREST, listO(3, SYM_MACROS_STELLA_rrARGS, SYM_MACROS_STELLA_STRING, NIL), NIL), KWD_MACROS_PUBLICp, SYM_MACROS_STELLA_TRUE, listO(3, SYM_MACROS_STELLA_LET, listO(3, cons(v1Name, listO(3, SYM_MACROS_STELLA_INTEGER, listO(4, SYM_MACROS_STELLA_VERBATIM, KWD_MACROS_COMMON_LISP, listO(3, internCommonLispSymbol("LENGTH"), SYM_MACROS_STELLA_rrARGS, NIL), NIL), NIL)), cons(v2Name, listO(3, listO(5, SYM_MACROS_STELLA_ARRAY, NIL, SYM_MACROS_STELLA_OF, SYM_MACROS_STELLA_STRING, NIL), listO(4, SYM_MACROS_STELLA_VERBATIM, KWD_MACROS_COMMON_LISP, listO(5, internCommonLispSymbol("MAKE-ARRAY"), listO(3, internCommonLispSymbol("LENGTH"), SYM_MACROS_STELLA_rrARGS, NIL), KWD_MACROS_INITIAL_CONTENTS, SYM_MACROS_STELLA_rrARGS, NIL), NIL), NIL)), NIL), startupfunctioncall->concatenate(body->concatenate(NIL, 0), 0)), NIL));
          }
          else if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_MACROS_JAVA) {
            return (listO(7, SYM_MACROS_STELLA_DEFUN, SYM_MACROS_STELLA_MAIN, cons(cons(v2Name, cons(listO(5, SYM_MACROS_STELLA_ARRAY, NIL, SYM_MACROS_STELLA_OF, SYM_MACROS_STELLA_STRING, NIL), NIL)), NIL), KWD_MACROS_PUBLICp, SYM_MACROS_STELLA_TRUE, listO(3, SYM_MACROS_STELLA_LET, cons(cons(v1Name, listO(3, SYM_MACROS_STELLA_INTEGER, listO(4, SYM_MACROS_STELLA_VERBATIM, KWD_MACROS_JAVA, wrapString(stringConcatenate(javaTranslateName(v2Name)->wrapperValue, ".length", 0)), NIL), NIL)), NIL), startupfunctioncall->concatenate(body->concatenate(NIL, 0), 0)), NIL));
          }
          else if ((oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_MACROS_CPP) ||
              ((oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_MACROS_CPP_STANDALONE) ||
               (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_MACROS_IDL))) {
            return (listO(6, SYM_MACROS_STELLA_DEFUN, listO(3, SYM_MACROS_STELLA_MAIN, SYM_MACROS_STELLA_INTEGER, NIL), listO(3, cons(v1Name, cons(SYM_MACROS_STELLA_INTEGER, NIL)), cons(v2Name, cons(listO(5, SYM_MACROS_STELLA_ARRAY, NIL, SYM_MACROS_STELLA_OF, SYM_MACROS_STELLA_STRING, NIL), NIL)), NIL), KWD_MACROS_PUBLICp, SYM_MACROS_STELLA_TRUE, startupfunctioncall->concatenate(body->concatenate(cons(listO(3, SYM_MACROS_STELLA_RETURN, wrapInteger(1), NIL), NIL), 0), 0)));
          }
          else {
          }
        }
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signalTranslationError();
          if (!(suppressWarningsP())) {
            printErrorContext(">> ERROR: ", STANDARD_ERROR);
            *(STANDARD_ERROR->nativeStream) << endl << " " << "`defmain' is not supported for " << "`" << oTRANSLATOROUTPUTLANGUAGEo.get() << "'" << "." << endl;
          }
        }
        return (NULL);
      }
    }
  }
}

void helpStartupMacros1() {
  {
    SYM_MACROS_STELLA_SETQ = ((Symbol*)(internRigidSymbolWrtModule("SETQ", NULL, 0)));
    SYM_MACROS_STELLA_CONS = ((Symbol*)(internRigidSymbolWrtModule("CONS", NULL, 0)));
    SYM_MACROS_STELLA_PUSHQ = ((Symbol*)(internRigidSymbolWrtModule("PUSHQ", NULL, 0)));
    SYM_MACROS_STELLA_PUSHQ_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("PUSHQ-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_VRLET = ((Symbol*)(internRigidSymbolWrtModule("VRLET", NULL, 0)));
    SYM_MACROS_STELLA_VALUE = ((Symbol*)(internRigidSymbolWrtModule("VALUE", NULL, 0)));
    SYM_MACROS_STELLA_REST = ((Symbol*)(internRigidSymbolWrtModule("REST", NULL, 0)));
    SYM_MACROS_STELLA_POPQ = ((Symbol*)(internRigidSymbolWrtModule("POPQ", NULL, 0)));
    SYM_MACROS_STELLA_POPQ_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("POPQ-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_SETF = ((Symbol*)(internRigidSymbolWrtModule("SETF", NULL, 0)));
    SYM_MACROS_STELLA_PUSHF = ((Symbol*)(internRigidSymbolWrtModule("PUSHF", NULL, 0)));
    SYM_MACROS_STELLA_PUSHF_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("PUSHF-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_WHEN = ((Symbol*)(internRigidSymbolWrtModule("WHEN", NULL, 0)));
    SYM_MACROS_STELLA_NOT = ((Symbol*)(internRigidSymbolWrtModule("NOT", NULL, 0)));
    SYM_MACROS_STELLA_MEMBERp = ((Symbol*)(internRigidSymbolWrtModule("MEMBER?", NULL, 0)));
    SYM_MACROS_STELLA_VARIABLE = ((Symbol*)(internRigidSymbolWrtModule("VARIABLE", NULL, 0)));
    SYM_MACROS_STELLA_PUSHQ_NEW = ((Symbol*)(internRigidSymbolWrtModule("PUSHQ-NEW", NULL, 0)));
    SYM_MACROS_STELLA_PUSHQ_NEW_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("PUSHQ-NEW-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_UNSTRINGIFY = ((Symbol*)(internRigidSymbolWrtModule("UNSTRINGIFY", NULL, 0)));
    SYM_MACROS_STELLA_STRING_QUOTE = ((Symbol*)(internRigidSymbolWrtModule("STRING-QUOTE", NULL, 0)));
    SYM_MACROS_STELLA_STRING_QUOTE_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("STRING-QUOTE-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_i = ((Symbol*)(internRigidSymbolWrtModule("+", NULL, 0)));
    SYM_MACROS_STELLA_ii = ((Symbol*)(internRigidSymbolWrtModule("++", NULL, 0)));
    SYM_MACROS_STELLA_ii_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("++-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA__ = ((Symbol*)(internRigidSymbolWrtModule("-", NULL, 0)));
    SYM_MACROS_STELLA___ = ((Symbol*)(internRigidSymbolWrtModule("--", NULL, 0)));
    SYM_MACROS_STELLA____EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("---EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_1i = ((Symbol*)(internRigidSymbolWrtModule("1+", NULL, 0)));
    SYM_MACROS_STELLA_1i_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("1+-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_1_ = ((Symbol*)(internRigidSymbolWrtModule("1-", NULL, 0)));
    SYM_MACROS_STELLA_1__EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("1--EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_DEFINEDp = ((Symbol*)(internRigidSymbolWrtModule("DEFINED?", NULL, 0)));
    SYM_MACROS_STELLA_SETQp = ((Symbol*)(internRigidSymbolWrtModule("SETQ?", NULL, 0)));
    SYM_MACROS_STELLA_SETQp_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("SETQ?-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_NULL = ((Symbol*)(internRigidSymbolWrtModule("NULL", NULL, 0)));
    SYM_MACROS_STELLA_CHOOSE = ((Symbol*)(internRigidSymbolWrtModule("CHOOSE", NULL, 0)));
    SYM_MACROS_STELLA_FIRST_DEFINED = ((Symbol*)(internRigidSymbolWrtModule("FIRST-DEFINED", NULL, 0)));
    SYM_MACROS_STELLA_FIRST_DEFINED_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("FIRST-DEFINED-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_AND = ((Symbol*)(internRigidSymbolWrtModule("AND", NULL, 0)));
    SYM_MACROS_STELLA_ALL_DEFINEDp = ((Symbol*)(internRigidSymbolWrtModule("ALL-DEFINED?", NULL, 0)));
    SYM_MACROS_STELLA_ALL_DEFINEDp_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("ALL-DEFINED?-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_EITHER = ((Symbol*)(internRigidSymbolWrtModule("EITHER", NULL, 0)));
    SYM_MACROS_STELLA_EITHER_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("EITHER-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_NIL = ((Symbol*)(internRigidSymbolWrtModule("NIL", NULL, 0)));
    SYM_MACROS_STELLA_FOREACH = ((Symbol*)(internRigidSymbolWrtModule("FOREACH", NULL, 0)));
    SYM_MACROS_STELLA_COLLECT = ((Symbol*)(internRigidSymbolWrtModule("COLLECT", NULL, 0)));
    SYM_MACROS_STELLA_INTO = ((Symbol*)(internRigidSymbolWrtModule("INTO", NULL, 0)));
    SYM_MACROS_STELLA_COLLECT_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("COLLECT-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_SPECIAL = ((Symbol*)(internRigidSymbolWrtModule("SPECIAL", NULL, 0)));
    SYM_MACROS_STELLA_oTRANSIENTOBJECTSpo = ((Symbol*)(internRigidSymbolWrtModule("*TRANSIENTOBJECTS?*", NULL, 0)));
    SYM_MACROS_STELLA_FALSE = ((Symbol*)(internRigidSymbolWrtModule("FALSE", NULL, 0)));
    SYM_MACROS_STELLA_WITH_PERMANENT_OBJECTS = ((Symbol*)(internRigidSymbolWrtModule("WITH-PERMANENT-OBJECTS", NULL, 0)));
    SYM_MACROS_STELLA_WITH_PERMANENT_OBJECTS_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("WITH-PERMANENT-OBJECTS-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_TRUE = ((Symbol*)(internRigidSymbolWrtModule("TRUE", NULL, 0)));
    SYM_MACROS_STELLA_WITH_TRANSIENT_OBJECTS = ((Symbol*)(internRigidSymbolWrtModule("WITH-TRANSIENT-OBJECTS", NULL, 0)));
    SYM_MACROS_STELLA_WITH_TRANSIENT_OBJECTS_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("WITH-TRANSIENT-OBJECTS-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_IGNORE = ((Symbol*)(internRigidSymbolWrtModule("IGNORE", NULL, 0)));
    SYM_MACROS_STELLA_IGNORE_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("IGNORE-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_ONLY_IF = ((Symbol*)(internRigidSymbolWrtModule("ONLY-IF", NULL, 0)));
    SYM_MACROS_STELLA_ONLY_IF_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("ONLY-IF-EXPANDER", NULL, 0)));
  }
}

void helpStartupMacros2() {
  {
    SYM_MACROS_STELLA_PHASE_TO_INTEGER = ((Symbol*)(internRigidSymbolWrtModule("PHASE-TO-INTEGER", NULL, 0)));
    SYM_MACROS_STELLA_PHASE_TO_INTEGER_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("PHASE-TO-INTEGER-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_IF_OUTPUT_LANGUAGE = ((Symbol*)(internRigidSymbolWrtModule("IF-OUTPUT-LANGUAGE", NULL, 0)));
    SYM_MACROS_STELLA_IF_OUTPUT_LANGUAGE_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("IF-OUTPUT-LANGUAGE-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_IF_STELLA_FEATURE = ((Symbol*)(internRigidSymbolWrtModule("IF-STELLA-FEATURE", NULL, 0)));
    SYM_MACROS_STELLA_IF_STELLA_FEATURE_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("IF-STELLA-FEATURE-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_oCONTEXTo = ((Symbol*)(internRigidSymbolWrtModule("*CONTEXT*", NULL, 0)));
    SYM_MACROS_STELLA_SAFETY = ((Symbol*)(internRigidSymbolWrtModule("SAFETY", NULL, 0)));
    SYM_MACROS_STELLA_EQLp = ((Symbol*)(internRigidSymbolWrtModule("EQL?", NULL, 0)));
    SYM_MACROS_STELLA_BASE_MODULE = ((Symbol*)(internRigidSymbolWrtModule("BASE-MODULE", NULL, 0)));
    SYM_MACROS_STELLA_CAST = ((Symbol*)(internRigidSymbolWrtModule("CAST", NULL, 0)));
    SYM_MACROS_STELLA_WORLD = ((Symbol*)(internRigidSymbolWrtModule("WORLD", NULL, 0)));
    SYM_MACROS_STELLA_oMODULEo = ((Symbol*)(internRigidSymbolWrtModule("*MODULE*", NULL, 0)));
    SYM_MACROS_STELLA_WITHIN_WORLD = ((Symbol*)(internRigidSymbolWrtModule("WITHIN-WORLD", NULL, 0)));
    SYM_MACROS_STELLA_WITHIN_WORLD_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("WITHIN-WORLD-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_WITHIN_CONTEXT = ((Symbol*)(internRigidSymbolWrtModule("WITHIN-CONTEXT", NULL, 0)));
    SYM_MACROS_STELLA_WITHIN_CONTEXT_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("WITHIN-CONTEXT-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_WITHIN_MODULE = ((Symbol*)(internRigidSymbolWrtModule("WITHIN-MODULE", NULL, 0)));
    SYM_MACROS_STELLA_WITHIN_MODULE_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("WITHIN-MODULE-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_IN = ((Symbol*)(internRigidSymbolWrtModule("IN", NULL, 0)));
    SYM_MACROS_STELLA_COERCE_aREST_TO_CONS = ((Symbol*)(internRigidSymbolWrtModule("COERCE-&REST-TO-CONS", NULL, 0)));
    SYM_MACROS_STELLA_COERCE_aREST_TO_CONS_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("COERCE-&REST-TO-CONS-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_oCURRENTSYSTEMDEFINITIONo = ((Symbol*)(internRigidSymbolWrtModule("*CURRENTSYSTEMDEFINITION*", NULL, 0)));
    SYM_MACROS_STELLA_GET_SYSTEM_DEFINITION = ((Symbol*)(internRigidSymbolWrtModule("GET-SYSTEM-DEFINITION", NULL, 0)));
    SYM_MACROS_STELLA_oCURRENTSYSTEMDEFINITIONSUBDIRECTORYo = ((Symbol*)(internRigidSymbolWrtModule("*CURRENTSYSTEMDEFINITIONSUBDIRECTORY*", NULL, 0)));
    SYM_MACROS_STELLA_DIRECTORY = ((Symbol*)(internRigidSymbolWrtModule("DIRECTORY", NULL, 0)));
    SYM_MACROS_STELLA_IF = ((Symbol*)(internRigidSymbolWrtModule("IF", NULL, 0)));
    SYM_MACROS_STELLA_PROGN = ((Symbol*)(internRigidSymbolWrtModule("PROGN", NULL, 0)));
    SYM_MACROS_STELLA_WARN = ((Symbol*)(internRigidSymbolWrtModule("WARN", NULL, 0)));
    SYM_MACROS_STELLA_EOL = ((Symbol*)(internRigidSymbolWrtModule("EOL", NULL, 0)));
    SYM_MACROS_STELLA_WITH_SYSTEM_DEFINITION = ((Symbol*)(internRigidSymbolWrtModule("WITH-SYSTEM-DEFINITION", NULL, 0)));
    SYM_MACROS_STELLA_WITH_SYSTEM_DEFINITION_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("WITH-SYSTEM-DEFINITION-EXPANDER", NULL, 0)));
    KWD_MACROS_STARTUP_SYSTEMp = ((Keyword*)(internRigidSymbolWrtModule("STARTUP-SYSTEM?", NULL, 2)));
    KWD_MACROS_PUBLICp = ((Keyword*)(internRigidSymbolWrtModule("PUBLIC?", NULL, 2)));
    KWD_MACROS_COMMON_LISP = ((Keyword*)(internRigidSymbolWrtModule("COMMON-LISP", NULL, 2)));
    SYM_MACROS_STELLA_DEFUN = ((Symbol*)(internRigidSymbolWrtModule("DEFUN", NULL, 0)));
    SYM_MACROS_STELLA_MAIN = ((Symbol*)(internRigidSymbolWrtModule("MAIN", NULL, 0)));
    KWD_MACROS_CPP = ((Keyword*)(internRigidSymbolWrtModule("CPP", NULL, 2)));
    KWD_MACROS_CPP_STANDALONE = ((Keyword*)(internRigidSymbolWrtModule("CPP-STANDALONE", NULL, 2)));
    KWD_MACROS_IDL = ((Keyword*)(internRigidSymbolWrtModule("IDL", NULL, 2)));
    SYM_MACROS_STELLA_INTEGER = ((Symbol*)(internRigidSymbolWrtModule("INTEGER", NULL, 0)));
    SYM_MACROS_STELLA_RETURN = ((Symbol*)(internRigidSymbolWrtModule("RETURN", NULL, 0)));
    KWD_MACROS_JAVA = ((Keyword*)(internRigidSymbolWrtModule("JAVA", NULL, 2)));
    SYM_MACROS_STELLA_ARRAY = ((Symbol*)(internRigidSymbolWrtModule("ARRAY", NULL, 0)));
    SYM_MACROS_STELLA_OF = ((Symbol*)(internRigidSymbolWrtModule("OF", NULL, 0)));
    SYM_MACROS_STELLA_STRING = ((Symbol*)(internRigidSymbolWrtModule("STRING", NULL, 0)));
    SYM_MACROS_STELLA_aREST = ((Symbol*)(internRigidSymbolWrtModule("&REST", NULL, 0)));
    SYM_MACROS_STELLA_rrARGS = ((Symbol*)(internRigidSymbolWrtModule("%%ARGS", NULL, 0)));
    SYM_MACROS_STELLA_LET = ((Symbol*)(internRigidSymbolWrtModule("LET", NULL, 0)));
    SYM_MACROS_STELLA_VERBATIM = ((Symbol*)(internRigidSymbolWrtModule("VERBATIM", NULL, 0)));
    KWD_MACROS_INITIAL_CONTENTS = ((Keyword*)(internRigidSymbolWrtModule("INITIAL-CONTENTS", NULL, 2)));
    SYM_MACROS_STELLA_DEFMAIN = ((Symbol*)(internRigidSymbolWrtModule("DEFMAIN", NULL, 0)));
    SYM_MACROS_STELLA_DEFMAIN_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("DEFMAIN-EXPANDER", NULL, 0)));
    SYM_MACROS_STELLA_STARTUP_MACROS = ((Symbol*)(internRigidSymbolWrtModule("STARTUP-MACROS", NULL, 0)));
    SYM_MACROS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(internRigidSymbolWrtModule("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void startupMacros() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (currentStartupTimePhaseP(2)) {
      helpStartupMacros1();
      helpStartupMacros2();
    }
    if (currentStartupTimePhaseP(6)) {
      finalizeClasses();
    }
    if (currentStartupTimePhaseP(7)) {
      defineFunctionObject("PUSHQ-EXPANDER", "(DEFUN (PUSHQ-EXPANDER OBJECT) ((VARIABLE SYMBOL) (VALUE OBJECT)) :DOCUMENTATION \"Push 'value' onto the cons list 'variable'.\")", ((cpp_function_code)(&pushqExpander)), NULL);
      defineFunctionObject("POPQ-EXPANDER", "(DEFUN (POPQ-EXPANDER OBJECT) ((VARIABLE SYMBOL)) :DOCUMENTATION \"Pops a value from the cons list 'variable'.\")", ((cpp_function_code)(&popqExpander)), NULL);
      defineFunctionObject("PUSHF-EXPANDER", "(DEFUN (PUSHF-EXPANDER OBJECT) ((PLACE CONS) (VALUE OBJECT)) :DOCUMENTATION \"Push 'value' onto the cons list 'place'.\")", ((cpp_function_code)(&pushfExpander)), NULL);
      defineFunctionObject("PUSHQ-NEW-EXPANDER", "(DEFUN (PUSHQ-NEW-EXPANDER OBJECT) ((VARIABLE SYMBOL) (VALUE OBJECT)) :DOCUMENTATION \"Push 'value' onto the cons list 'variable', if its\nnot there already.\")", ((cpp_function_code)(&pushqNewExpander)), NULL);
      defineFunctionObject("STRING-QUOTE-EXPANDER", "(DEFUN (STRING-QUOTE-EXPANDER OBJECT) ((TREE OBJECT)) :DOCUMENTATION \"Return a parse tree that evaluates to 'tree' at\nrun-time.  'tree' is encoded as a string within the parse tree.\")", ((cpp_function_code)(&stringQuoteExpander)), NULL);
      defineFunctionObject("++-EXPANDER", "(DEFUN (++-EXPANDER OBJECT) ((PLACE OBJECT) |&BODY| (INCREMENT CONS)) :DOCUMENTATION \"Increment the value of `place' and return the result.\n`place' can be either a variable name or a slot reference.  Increment by\nthe optional `increment' (which can be a float) or 1 otherwise.\")", ((cpp_function_code)(&IIExpander)), NULL);
      defineFunctionObject("---EXPANDER", "(DEFUN (---EXPANDER OBJECT) ((PLACE OBJECT) |&BODY| (DECREMENT CONS)) :DOCUMENTATION \"Decrement the value of `place' and return the result.\n`place' can be either a variable name or a slot reference.  Decrement by\nthe optional `decrement' (which can be a float) or 1 otherwise.\")", ((cpp_function_code)(&Expander)), NULL);
      defineFunctionObject("1+-EXPANDER", "(DEFUN (1+-EXPANDER OBJECT) ((EXPRESSION OBJECT)) :DOCUMENTATION \"Add 1 to 'expression' and return the result.\")", ((cpp_function_code)(&oneIExpander)), NULL);
      defineFunctionObject("1--EXPANDER", "(DEFUN (1--EXPANDER OBJECT) ((EXPRESSION OBJECT)) :DOCUMENTATION \"Subtract 1 from 'expression' and return the result.\")", ((cpp_function_code)(&oneExpander)), NULL);
      defineFunctionObject("SETQ?-EXPANDER", "(DEFUN (SETQ?-EXPANDER OBJECT) ((VARIABLE SYMBOL) (EXPRESSION CONS)) :DOCUMENTATION \"Assign 'variable' the result of evaluating 'expression', \nand return TRUE if 'expression' is not NULL else return FALSE.\")", ((cpp_function_code)(&setqPExpander)), NULL);
      defineFunctionObject("FIRST-DEFINED-EXPANDER", "(DEFUN (FIRST-DEFINED-EXPANDER OBJECT) (|&BODY| (FORMS CONS)) :DOCUMENTATION \"Return the result of the first form in `forms' whose value is defined\nor NULL otherwise.\")", ((cpp_function_code)(&firstDefinedExpander)), NULL);
      defineFunctionObject("ALL-DEFINED?-EXPANDER", "(DEFUN (ALL-DEFINED?-EXPANDER OBJECT) (|&BODY| (FORMS CONS)) :DOCUMENTATION \"Evaluate each of the forms in 'forms', and\nreturn TRUE if none of them are NULL.\")", ((cpp_function_code)(&allDefinedPExpander)), NULL);
      defineFunctionObject("EITHER-EXPANDER", "(DEFUN (EITHER-EXPANDER OBJECT) ((VALUE1 OBJECT) (VALUE2 OBJECT)) :DOCUMENTATION \"If 'value1' is defined, return that, else return 'value2'.\")", ((cpp_function_code)(&eitherExpander)), NULL);
      defineFunctionObject("COLLECT-EXPANDER", "(DEFUN (COLLECT-EXPANDER OBJECT) ((COLLECTVARIABLE SYMBOL) |&BODY| (BODY CONS)) :DOCUMENTATION \"Use a VRLET to collect values.  Input has the form\n'(collect <x> in <expression> where (<test> <x>))'.\")", ((cpp_function_code)(&collectExpander)), NULL);
      defineFunctionObject("WITH-PERMANENT-OBJECTS-EXPANDER", "(DEFUN (WITH-PERMANENT-OBJECTS-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :DOCUMENTATION \"Allocate 'permanent' (as opposed to 'transient')\nobjects within the scope of this declaration.\" :PUBLIC? TRUE)", ((cpp_function_code)(&withPermanentObjectsExpander)), NULL);
      defineFunctionObject("WITH-TRANSIENT-OBJECTS-EXPANDER", "(DEFUN (WITH-TRANSIENT-OBJECTS-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :DOCUMENTATION \"Allocate 'transient' (as opposed to 'permanent')\nobjects within the scope of this declaration.\nCAUTION: The default assumption is the allocation of permanent objects.\nThe scope of `with-transient-objects' should be as small as possible, and\nthe user has to make sure that code that wasn't explicitly written to\naccount for transient objects will continue to work correctly.\" :PUBLIC? TRUE)", ((cpp_function_code)(&withTransientObjectsExpander)), NULL);
      defineFunctionObject("IGNORE-EXPANDER", "(DEFUN (IGNORE-EXPANDER OBJECT) (|&BODY| (VARIABLES CONS)) :DOCUMENTATION \"Ignore unused 'variables' with NoOp 'setq' statements.\")", ((cpp_function_code)(&ignoreExpander)), NULL);
      defineFunctionObject("ONLY-IF-EXPANDER", "(DEFUN (ONLY-IF-EXPANDER OBJECT) ((TEST OBJECT) (EXPRESSION OBJECT)) :DOCUMENTATION \"If 'test' is TRUE, return the result of evaluating \n'expression'.\")", ((cpp_function_code)(&onlyIfExpander)), NULL);
      defineFunctionObject("PHASE-TO-INTEGER-EXPANDER", "(DEFUN (PHASE-TO-INTEGER-EXPANDER OBJECT) ((STARTUPTIMEPHASE KEYWORD)) :DOCUMENTATION \"Expands into the integer representing 'startupTimePhase'.\")", ((cpp_function_code)(&phaseToIntegerExpander)), NULL);
      defineFunctionObject("IF-OUTPUT-LANGUAGE-EXPANDER", "(DEFUN (IF-OUTPUT-LANGUAGE-EXPANDER OBJECT) ((LANGUAGE KEYWORD) (THENFORM OBJECT) (ELSEFORM OBJECT)) :DOCUMENTATION \"Expand to 'thenForm' if the current translator output\nlanguage equals 'language'.  Otherwise, expand to 'elseForm'.  This can\nbe used to conditionally translate Stella code.\")", ((cpp_function_code)(&ifOutputLanguageExpander)), NULL);
      defineFunctionObject("IF-STELLA-FEATURE-EXPANDER", "(DEFUN (IF-STELLA-FEATURE-EXPANDER OBJECT) ((FEATURE KEYWORD) (THENFORM OBJECT) (ELSEFORM OBJECT)) :DOCUMENTATION \"Expand to 'thenForm' if 'feature' is a currently enabled\nSTELLA environment feature.  Otherwise, expand to 'elseForm'.  This can\nbe used to conditionally translate Stella code.\")", ((cpp_function_code)(&ifStellaFeatureExpander)), NULL);
      defineFunctionObject("WITHIN-WORLD-EXPANDER", "(DEFUN (WITHIN-WORLD-EXPANDER OBJECT) ((WORLDFORM OBJECT) |&BODY| (BODY CONS)) :DOCUMENTATION \"Execute `body' within the world resulting from `worldForm'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&withinWorldExpander)), NULL);
      defineFunctionObject("WITHIN-CONTEXT-EXPANDER", "(DEFUN (WITHIN-CONTEXT-EXPANDER OBJECT) ((CONTEXTFORM OBJECT) |&BODY| (BODY CONS)) :DOCUMENTATION \"Execute `body' within the context resulting from `contextForm'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&withinContextExpander)), NULL);
      defineFunctionObject("WITHIN-MODULE-EXPANDER", "(DEFUN (WITHIN-MODULE-EXPANDER OBJECT) ((MODULEFORM OBJECT) |&BODY| (BODY CONS)) :DOCUMENTATION \"Execute `body' within the module resulting from `moduleForm'.\n`*module*' is an acceptable `moduleForm'.  It will locally rebind \n`*module*' and `*context*' and shield the outer bindings from changes.\" :PUBLIC? TRUE)", ((cpp_function_code)(&withinModuleExpander)), NULL);
      defineFunctionObject(" |COERCE-&REST-TO-CONS-EXPANDER|", "(DEFUN (|COERCE-&REST-TO-CONS-EXPANDER| OBJECT) ((RESTVARIABLE SYMBOL)) :DOCUMENTATION \"Coerce the argument list variable `restVariable' into a CONS\nlist containing all its elements (uses argument list iteration to do so).  If\n`restVariable' already is a CONS due to argument listification, this is a no-op.\" :PUBLIC? TRUE)", ((cpp_function_code)(&COERCE_AREST_TO_CONS_EXPANDER)), NULL);
      defineFunctionObject("WITH-SYSTEM-DEFINITION-EXPANDER", "(DEFUN (WITH-SYSTEM-DEFINITION-EXPANDER OBJECT) ((SYSTEMNAMEEXPRESSION OBJECT) |&BODY| (BODY CONS)) :DOCUMENTATION \"Set *currentSystemDefinition* to the system definition named 'system'.\nSet *currentSystemDefinitionSubdirectory* to match. Execute 'body' within\nthat scope.\" :PUBLIC? TRUE)", ((cpp_function_code)(&withSystemDefinitionExpander)), NULL);
      defineFunctionObject("DEFMAIN-EXPANDER", "(DEFUN (DEFMAIN-EXPANDER OBJECT) ((VARLIST CONS) |&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Defines a function called MAIN which will have the appropriate\nsignature for the target translation language.  The signature will be:\n  C++:   public static int main (int v1, char** v2) {<body>}\n}}  Java:  public static void main (String [] v2) {<body>}\n  Lisp:  (defun main (&rest args) <body>)\nThe argument `varList' must have two symbols, which will be the names for the\nINTEGER argument count and an array of STRINGs with the argument values.  It\ncan also be empty to indicate that no command line arguments will be handled.\nThe startup function for the containing system will automatically be called\nbefore `body' is executed unless the option :STARTUP-SYSTEM? was supplied as\nFALSE.  There can only be one DEFMAIN per module.\")", ((cpp_function_code)(&defmainExpander)), NULL);
      defineFunctionObject("STARTUP-MACROS", "(DEFUN STARTUP-MACROS () :PUBLIC? TRUE)", ((cpp_function_code)(&startupMacros)), NULL);
      { MethodSlot* function = lookupFunction(SYM_MACROS_STELLA_STARTUP_MACROS);

        setDynamicSlotValue(function->dynamicSlots, SYM_MACROS_STELLA_METHOD_STARTUP_CLASSNAME, wrapString("Startup-Macros"), NULL_STRING_WRAPPER);
      }
    }
    if (currentStartupTimePhaseP(8)) {
      finalizeSlots();
      cleanupUnfinalizedClasses();
    }
    if (currentStartupTimePhaseP(9)) {
      storeMacro(SYM_MACROS_STELLA_PUSHQ, SYM_MACROS_STELLA_PUSHQ_EXPANDER, ((cpp_function_code)(&pushqExpander)));
      storeMacro(SYM_MACROS_STELLA_POPQ, SYM_MACROS_STELLA_POPQ_EXPANDER, ((cpp_function_code)(&popqExpander)));
      storeMacro(SYM_MACROS_STELLA_PUSHF, SYM_MACROS_STELLA_PUSHF_EXPANDER, ((cpp_function_code)(&pushfExpander)));
      storeMacro(SYM_MACROS_STELLA_PUSHQ_NEW, SYM_MACROS_STELLA_PUSHQ_NEW_EXPANDER, ((cpp_function_code)(&pushqNewExpander)));
      storeMacro(SYM_MACROS_STELLA_STRING_QUOTE, SYM_MACROS_STELLA_STRING_QUOTE_EXPANDER, ((cpp_function_code)(&stringQuoteExpander)));
      storeMacro(SYM_MACROS_STELLA_ii, SYM_MACROS_STELLA_ii_EXPANDER, ((cpp_function_code)(&IIExpander)));
      storeMacro(SYM_MACROS_STELLA___, SYM_MACROS_STELLA____EXPANDER, ((cpp_function_code)(&Expander)));
      storeMacro(SYM_MACROS_STELLA_1i, SYM_MACROS_STELLA_1i_EXPANDER, ((cpp_function_code)(&oneIExpander)));
      storeMacro(SYM_MACROS_STELLA_1_, SYM_MACROS_STELLA_1__EXPANDER, ((cpp_function_code)(&oneExpander)));
      storeMacro(SYM_MACROS_STELLA_SETQp, SYM_MACROS_STELLA_SETQp_EXPANDER, ((cpp_function_code)(&setqPExpander)));
      storeMacro(SYM_MACROS_STELLA_FIRST_DEFINED, SYM_MACROS_STELLA_FIRST_DEFINED_EXPANDER, ((cpp_function_code)(&firstDefinedExpander)));
      storeMacro(SYM_MACROS_STELLA_ALL_DEFINEDp, SYM_MACROS_STELLA_ALL_DEFINEDp_EXPANDER, ((cpp_function_code)(&allDefinedPExpander)));
      storeMacro(SYM_MACROS_STELLA_EITHER, SYM_MACROS_STELLA_EITHER_EXPANDER, ((cpp_function_code)(&eitherExpander)));
      storeMacro(SYM_MACROS_STELLA_COLLECT, SYM_MACROS_STELLA_COLLECT_EXPANDER, ((cpp_function_code)(&collectExpander)));
      storeMacro(SYM_MACROS_STELLA_WITH_PERMANENT_OBJECTS, SYM_MACROS_STELLA_WITH_PERMANENT_OBJECTS_EXPANDER, ((cpp_function_code)(&withPermanentObjectsExpander)));
      storeMacro(SYM_MACROS_STELLA_WITH_TRANSIENT_OBJECTS, SYM_MACROS_STELLA_WITH_TRANSIENT_OBJECTS_EXPANDER, ((cpp_function_code)(&withTransientObjectsExpander)));
      storeMacro(SYM_MACROS_STELLA_IGNORE, SYM_MACROS_STELLA_IGNORE_EXPANDER, ((cpp_function_code)(&ignoreExpander)));
      storeMacro(SYM_MACROS_STELLA_ONLY_IF, SYM_MACROS_STELLA_ONLY_IF_EXPANDER, ((cpp_function_code)(&onlyIfExpander)));
      storeMacro(SYM_MACROS_STELLA_PHASE_TO_INTEGER, SYM_MACROS_STELLA_PHASE_TO_INTEGER_EXPANDER, ((cpp_function_code)(&phaseToIntegerExpander)));
      storeMacro(SYM_MACROS_STELLA_IF_OUTPUT_LANGUAGE, SYM_MACROS_STELLA_IF_OUTPUT_LANGUAGE_EXPANDER, ((cpp_function_code)(&ifOutputLanguageExpander)));
      storeMacro(SYM_MACROS_STELLA_IF_STELLA_FEATURE, SYM_MACROS_STELLA_IF_STELLA_FEATURE_EXPANDER, ((cpp_function_code)(&ifStellaFeatureExpander)));
      storeMacro(SYM_MACROS_STELLA_WITHIN_WORLD, SYM_MACROS_STELLA_WITHIN_WORLD_EXPANDER, ((cpp_function_code)(&withinWorldExpander)));
      storeMacro(SYM_MACROS_STELLA_WITHIN_CONTEXT, SYM_MACROS_STELLA_WITHIN_CONTEXT_EXPANDER, ((cpp_function_code)(&withinContextExpander)));
      storeMacro(SYM_MACROS_STELLA_WITHIN_MODULE, SYM_MACROS_STELLA_WITHIN_MODULE_EXPANDER, ((cpp_function_code)(&withinModuleExpander)));
      storeMacro(SYM_MACROS_STELLA_COERCE_aREST_TO_CONS, SYM_MACROS_STELLA_COERCE_aREST_TO_CONS_EXPANDER, ((cpp_function_code)(&COERCE_AREST_TO_CONS_EXPANDER)));
      storeMacro(SYM_MACROS_STELLA_WITH_SYSTEM_DEFINITION, SYM_MACROS_STELLA_WITH_SYSTEM_DEFINITION_EXPANDER, ((cpp_function_code)(&withSystemDefinitionExpander)));
      storeMacro(SYM_MACROS_STELLA_DEFMAIN, SYM_MACROS_STELLA_DEFMAIN_EXPANDER, ((cpp_function_code)(&defmainExpander)));
    }
  }
}

Symbol* SYM_MACROS_STELLA_SETQ = NULL;

Symbol* SYM_MACROS_STELLA_CONS = NULL;

Symbol* SYM_MACROS_STELLA_PUSHQ = NULL;

Symbol* SYM_MACROS_STELLA_PUSHQ_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_VRLET = NULL;

Symbol* SYM_MACROS_STELLA_VALUE = NULL;

Symbol* SYM_MACROS_STELLA_REST = NULL;

Symbol* SYM_MACROS_STELLA_POPQ = NULL;

Symbol* SYM_MACROS_STELLA_POPQ_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_SETF = NULL;

Symbol* SYM_MACROS_STELLA_PUSHF = NULL;

Symbol* SYM_MACROS_STELLA_PUSHF_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_WHEN = NULL;

Symbol* SYM_MACROS_STELLA_NOT = NULL;

Symbol* SYM_MACROS_STELLA_MEMBERp = NULL;

Symbol* SYM_MACROS_STELLA_VARIABLE = NULL;

Symbol* SYM_MACROS_STELLA_PUSHQ_NEW = NULL;

Symbol* SYM_MACROS_STELLA_PUSHQ_NEW_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_UNSTRINGIFY = NULL;

Symbol* SYM_MACROS_STELLA_STRING_QUOTE = NULL;

Symbol* SYM_MACROS_STELLA_STRING_QUOTE_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_i = NULL;

Symbol* SYM_MACROS_STELLA_ii = NULL;

Symbol* SYM_MACROS_STELLA_ii_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA__ = NULL;

Symbol* SYM_MACROS_STELLA___ = NULL;

Symbol* SYM_MACROS_STELLA____EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_1i = NULL;

Symbol* SYM_MACROS_STELLA_1i_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_1_ = NULL;

Symbol* SYM_MACROS_STELLA_1__EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_DEFINEDp = NULL;

Symbol* SYM_MACROS_STELLA_SETQp = NULL;

Symbol* SYM_MACROS_STELLA_SETQp_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_NULL = NULL;

Symbol* SYM_MACROS_STELLA_CHOOSE = NULL;

Symbol* SYM_MACROS_STELLA_FIRST_DEFINED = NULL;

Symbol* SYM_MACROS_STELLA_FIRST_DEFINED_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_AND = NULL;

Symbol* SYM_MACROS_STELLA_ALL_DEFINEDp = NULL;

Symbol* SYM_MACROS_STELLA_ALL_DEFINEDp_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_EITHER = NULL;

Symbol* SYM_MACROS_STELLA_EITHER_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_NIL = NULL;

Symbol* SYM_MACROS_STELLA_FOREACH = NULL;

Symbol* SYM_MACROS_STELLA_COLLECT = NULL;

Symbol* SYM_MACROS_STELLA_INTO = NULL;

Symbol* SYM_MACROS_STELLA_COLLECT_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_SPECIAL = NULL;

Symbol* SYM_MACROS_STELLA_oTRANSIENTOBJECTSpo = NULL;

Symbol* SYM_MACROS_STELLA_FALSE = NULL;

Symbol* SYM_MACROS_STELLA_WITH_PERMANENT_OBJECTS = NULL;

Symbol* SYM_MACROS_STELLA_WITH_PERMANENT_OBJECTS_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_TRUE = NULL;

Symbol* SYM_MACROS_STELLA_WITH_TRANSIENT_OBJECTS = NULL;

Symbol* SYM_MACROS_STELLA_WITH_TRANSIENT_OBJECTS_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_IGNORE = NULL;

Symbol* SYM_MACROS_STELLA_IGNORE_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_ONLY_IF = NULL;

Symbol* SYM_MACROS_STELLA_ONLY_IF_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_PHASE_TO_INTEGER = NULL;

Symbol* SYM_MACROS_STELLA_PHASE_TO_INTEGER_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_IF_OUTPUT_LANGUAGE = NULL;

Symbol* SYM_MACROS_STELLA_IF_OUTPUT_LANGUAGE_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_IF_STELLA_FEATURE = NULL;

Symbol* SYM_MACROS_STELLA_IF_STELLA_FEATURE_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_oCONTEXTo = NULL;

Symbol* SYM_MACROS_STELLA_SAFETY = NULL;

Symbol* SYM_MACROS_STELLA_EQLp = NULL;

Symbol* SYM_MACROS_STELLA_BASE_MODULE = NULL;

Symbol* SYM_MACROS_STELLA_CAST = NULL;

Symbol* SYM_MACROS_STELLA_WORLD = NULL;

Symbol* SYM_MACROS_STELLA_oMODULEo = NULL;

Symbol* SYM_MACROS_STELLA_WITHIN_WORLD = NULL;

Symbol* SYM_MACROS_STELLA_WITHIN_WORLD_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_WITHIN_CONTEXT = NULL;

Symbol* SYM_MACROS_STELLA_WITHIN_CONTEXT_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_WITHIN_MODULE = NULL;

Symbol* SYM_MACROS_STELLA_WITHIN_MODULE_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_IN = NULL;

Symbol* SYM_MACROS_STELLA_COERCE_aREST_TO_CONS = NULL;

Symbol* SYM_MACROS_STELLA_COERCE_aREST_TO_CONS_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_oCURRENTSYSTEMDEFINITIONo = NULL;

Symbol* SYM_MACROS_STELLA_GET_SYSTEM_DEFINITION = NULL;

Symbol* SYM_MACROS_STELLA_oCURRENTSYSTEMDEFINITIONSUBDIRECTORYo = NULL;

Symbol* SYM_MACROS_STELLA_DIRECTORY = NULL;

Symbol* SYM_MACROS_STELLA_IF = NULL;

Symbol* SYM_MACROS_STELLA_PROGN = NULL;

Symbol* SYM_MACROS_STELLA_WARN = NULL;

Symbol* SYM_MACROS_STELLA_EOL = NULL;

Symbol* SYM_MACROS_STELLA_WITH_SYSTEM_DEFINITION = NULL;

Symbol* SYM_MACROS_STELLA_WITH_SYSTEM_DEFINITION_EXPANDER = NULL;

Keyword* KWD_MACROS_STARTUP_SYSTEMp = NULL;

Keyword* KWD_MACROS_PUBLICp = NULL;

Keyword* KWD_MACROS_COMMON_LISP = NULL;

Symbol* SYM_MACROS_STELLA_DEFUN = NULL;

Symbol* SYM_MACROS_STELLA_MAIN = NULL;

Keyword* KWD_MACROS_CPP = NULL;

Keyword* KWD_MACROS_CPP_STANDALONE = NULL;

Keyword* KWD_MACROS_IDL = NULL;

Symbol* SYM_MACROS_STELLA_INTEGER = NULL;

Symbol* SYM_MACROS_STELLA_RETURN = NULL;

Keyword* KWD_MACROS_JAVA = NULL;

Symbol* SYM_MACROS_STELLA_ARRAY = NULL;

Symbol* SYM_MACROS_STELLA_OF = NULL;

Symbol* SYM_MACROS_STELLA_STRING = NULL;

Symbol* SYM_MACROS_STELLA_aREST = NULL;

Symbol* SYM_MACROS_STELLA_rrARGS = NULL;

Symbol* SYM_MACROS_STELLA_LET = NULL;

Symbol* SYM_MACROS_STELLA_VERBATIM = NULL;

Keyword* KWD_MACROS_INITIAL_CONTENTS = NULL;

Symbol* SYM_MACROS_STELLA_DEFMAIN = NULL;

Symbol* SYM_MACROS_STELLA_DEFMAIN_EXPANDER = NULL;

Symbol* SYM_MACROS_STELLA_STARTUP_MACROS = NULL;

Symbol* SYM_MACROS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
